searchNodes=[{"doc":"The MonoRepo library offers you a pattern of developing your applications in a mono repository and a set of functions to do it easily. With mono repo pattern your umbrella applications can nest other mono or umbrella applications. The library is split in 3 modules named after application lifecycle phases: Build, Test, Release. For specific functionality documentation please refer to corresponding module docs. Root application It is the top-level application which must be an umbrella application. It is recommended to use empty string(&quot;&quot;) as :apps_path value to avoid dependencies decla ration issues. Parent application Any linked application that is closer to the project&#39;s root application is a parent application. /1 functions from MonoRepo.Build module can be used to target them if you want to keep common configuration or dependencies in parent&#39;s folder instead of the root one. Child application Any linked application that is further from the project&#39;s root application is a child application. Child applications can be developed as standalone apps except for the mix.exs could be set to use root&#39;s/parent&#39;s folders for keeping dependencies and build artefacts. Applications should be nested in parent&#39;s apps folders. For example: &quot;app0/ apps/app1/apps/app2&quot;. Using MonoRepo is not a part of the mix application, so it is not loaded at the moment when mix.exs is being read. There are two ways of fixing that: append MonoRepo&#39;s beam files&#39; path before making calls to it. Use Code module: true = Code.append_path(&quot;_build/dev/lib/mono_repo/ebin&quot;) Be sure the :mono_repo dependency is compiled before using any of it&#39;s modules . Put MonoRepo codes under Mix.Project.MonoRepo namespace, compile them, copy to mix lib folder(/usr/lib/elixir/lib/mix/ebin/ on my machine) and add alien modules to modules list in mix.app. After those manipulations, you would be able to import MonoRepo modules. This is dirty and unrecommended but very convinient and effective. Build Most of the times you&#39;ll want to keep all the dependencies and build artefacts at one place to avoid duplicating, version mismatching and curly paths manual searches. You can do so by: Assigning :build_path to MonoRepo.Build.build_path/0. Assigning :deps_path to MonoRepo.Build.build_deps_path/0. Assigning &quot;:lockfile&quot; to MonoRepo.Build.build_lockfile_path/0. Umbrella&#39;s documentation recommends keeping child application configuration in parent&#39;s one. If you feel like doing that, you can use MonoRepo.Build.build_config_path/0 or MonoRepo.Build.build_config_path/1. If your application is deeply nested, it can be tedious and error prone to type dots in the path to a parent app. Consider typing build_config_path() instead of &quot;../../../../../../config/config.exs&quot;. Personally I prefer to have application&#39;s configuration at hands. For releasing a build a separate configuration file is declared. Test It is possible to run all children&#39;s tests at once by proper using MonoRepo.Test&#39;s module&#39;s functions. The general recomendation is to use the root level project folder mainly for testing purposes. Build a united configuration by running MonoRepo.Test.build_config_files(). This will traverse the root application&#39;s tree and copy all config.exs and test.exs configuration files into two single files at ./config path. MonoRepo.Test.build_test_paths/0. Assign :test_paths key in the root application to MonoRepo.Test.build_test_paths/0. Assign :deps key to MonoRepo.Test.build_deps/0. If you need to define other dependencies, you can do it in a standard way within deps/0 function and concatenate both lists with ++ operator. Now run &#39;mix test&#39; to run all tests in all child applications. Step one level into apps folder, repeat the setup and run it again to get -1 level of applications to test. If you use :apps_path key in your Mix.Project declaration, testing won&#39;t be available, even direct calls to test files won&#39;t work. The workaround is to have a separate mix.exs file or to comment out the :apps_path line meanwhile testing. Testing requires applications and their dependencies to be started. That&#39;s why we use build_deps/0 as value of :deps key. Release Release does not hack or alter Mix.Release functionality. To make a release you must describe it in rel/definitions.exs following instructions in MonoRepo.Release module. rel/&quot;release name&quot;.exs must hold a regular application configuration specific for your release. rel/mix.exs must declare a MixProject suitable for releasing: :deps key must be set to MonoRepo.Release.build_deps/0 :releases - to MonoRepo.Release.build_releases/0 :config_path to MonoRepo.Release.build_config_path/0 as a compile-time configuration. This way your app file will get only necessary arguments and your VM won&#39;t get rebooted after configuration loading unless you use releases.exs configuration as well. Keep mono_repo as a dependency in release mix project file to avoid erasing it&#39;s beam files. If you need run-time configuration, release.exs will be searched in config folder and loaded by Mix.Release. If you&#39;ve got any other dependencies, you can define them in deps/0 as usual and concatenate that list on release dependencies, like that: deps: deps() ++ build_deps(). MonoRepo.Release uses release name command-line argument so make sure you don&#39;t put any switches or arguments before the name. mix.exs sample: ... import MonoRepo.Release ... def project do [ ... config_path: build_config_path(), deps: build_deps() ++ deps(), releases: build_releases() ] end defp deps do [ {:mono_repo, path: &quot;../mono_repo&quot;, only: [:dev, :prod], runtime: false}, ] end ...","ref":"MonoRepo.html","title":"MonoRepo","type":"module"},{"doc":"Returns a version read from &quot;version&quot; file in application&#39;s root directory. The aim here is to avoid forgetting updating version in neither development mix.exs nor rel/mix.exs. The dev mix.exs should read from the file using this function as well.","ref":"MonoRepo.html#version/0","title":"MonoRepo.version/0","type":"function"},{"doc":"Child is an application atom which position is lower in a mono repo hierachy.","ref":"MonoRepo.html#t:child/0","title":"MonoRepo.child/0","type":"type"},{"doc":"Parent is an application atom which position is higher in a mono repo hierachy .","ref":"MonoRepo.html#t:parent/0","title":"MonoRepo.parent/0","type":"type"},{"doc":"Root is an application atom which position is the highest in a mono repo hierachy.","ref":"MonoRepo.html#t:root/0","title":"MonoRepo.root/0","type":"type"},{"doc":"This module holds a set of search path functions for convinient building of paths to parent applications. These functions take a parent application&#39;s name as argument and return a string of unexpanded path to parent application relative to caller&#39;s directory . For example: if root_app includes child_app as tier 1 child then calling any function will return &quot;../../specific_path&quot;.","ref":"MonoRepo.Build.html","title":"MonoRepo.Build","type":"module"},{"doc":"Searches for the root or parent application and returns a path to it&#39;s configuration directory. The first argument is an application&#39;s name as a string, the second one is a path to be appended to parent&#39;s application path. Raises RuntimeError in case the path can&#39;t be resolved.","ref":"MonoRepo.Build.html#build_config_path/2","title":"MonoRepo.Build.build_config_path/2","type":"function"},{"doc":"Searches for the root or parent application and returns a path to it&#39;s dependencies directory. Builds a path to root/parent application&#39;s dependencies directory(deps) or raises a RuntimeError. The first argument is an application&#39;s name as a string, the second one is a path to be appended to parent&#39;s application path.","ref":"MonoRepo.Build.html#build_deps_path/2","title":"MonoRepo.Build.build_deps_path/2","type":"function"},{"doc":"Searches for the root or parent application and returns a path to it&#39;s lockfile path directory. Builds a path to root/parent application&#39;s lockfile(mix.lock) or raises a RuntimeError. The first argument is an application&#39;s name as a string, the second one is a path to be appended to parent&#39;s application path.","ref":"MonoRepo.Build.html#build_lockfile_path/2","title":"MonoRepo.Build.build_lockfile_path/2","type":"function"},{"doc":"Searches for the root or parent application and returns a path to it&#39;s build directory. Builds a path to parent application&#39;s build directory(_build) or raises a RuntimeError. The first argument is an application&#39;s name as a string, the second one is a path to be appended to parent&#39;s application path.","ref":"MonoRepo.Build.html#build_path/2","title":"MonoRepo.Build.build_path/2","type":"function"},{"doc":"Release holds functions for building dependencies and releases list from custom configuration. For proper work you need to create a separate mix.exs, definition.exs and &quot;release name&quot;.exs files in rel folder in project&#39;s root. Mix project configurations for development and for releasing with MonoRepo pattern are quite different, so we create a separate mix.exs for releases. definition.exs describes our releases using Config module. Root key of a configuration must be a release name. :applications attribute should be a list of applications&#39; names in a specific format which will be passed by this module&#39;s functions. The valid format for app name is path/to/app. Be sure to avoid mentioning apps folders in your paths. For example if your app is nested like root_app/apps/app1/apps/app2, you should refer to it as app1/app2. All the other release options found in Mix.Release documentation can be passed as is. The last file we need is a &quot;release name&quot;.exs. It&#39;s your compile-time configuration for your release. It should be used for :config_path attribute of mix project configuration. This way you can keep release-specific configuration nice and tidy and do not load unnecessary configuration attributes. To make a release use MIX_EXS environment variable set to the release MixProject file. MonoRepo.Release uses release name command-line argument so make sure you don&#39;t put any switches or arguments before the name. Usage: MIX_EXS=rel/mix.exs mix release set0 definitions.exs sample: import Config config :set0, applications: [&quot;child0&quot;, &quot;child1/child2&quot;], strip_beams: false config :set1, applications: ~w(child3/child4/child5 child6/child7)","ref":"MonoRepo.Release.html","title":"MonoRepo.Release","type":"module"},{"doc":"Returns release-specific configuration file path. It relies on release name from passed arguments. So NIX_EXS=rel/mix.exs mix release set0 will assign config_path to rel/set0.exs in case mentioned mix.exs uses this module&#39;s config_path function.","ref":"MonoRepo.Release.html#build_config_path/0","title":"MonoRepo.Release.build_config_path/0","type":"function"},{"doc":"Builds a list of dependencies from rel/definitions.exs. All dependencies will get released the same way your main application will. This way you can add any application or nested application at any level to your final release.","ref":"MonoRepo.Release.html#build_deps/0","title":"MonoRepo.Release.build_deps/0","type":"function"},{"doc":"Builds a standard releases list with all applications set to permanent mode. Other modes are not yet supported. Requires a rel/definitions.exs to read release setup. This function must always be used since there is no default release, so building one requires a name and that requires a release declaration.","ref":"MonoRepo.Release.html#build_releases/0","title":"MonoRepo.Release.build_releases/0","type":"function"},{"doc":"This module holds a set of functions to build test paths for your mix project configuration. Instead of manually creating lists of folded applications` paths like &quot;apps/app0/apps/app1/apps/app2/test&quot; use functions from this module.","ref":"MonoRepo.Test.html","title":"MonoRepo.Test","type":"module"},{"doc":"Assembles all configuration files of all application&#39;s children into one. Runs this procedure for cofig.exs and test.exs files found in children&#39;s config folders and writes assembled config.exs and test.exs to application&#39;s root folder, to conig subfolder.","ref":"MonoRepo.Test.html#build_config_files/0","title":"MonoRepo.Test.build_config_files/0","type":"function"},{"doc":"Builds dependencies list to support testing. This function MUST be used together with build_test_paths/0 in order to start all applications before tests evaluation.","ref":"MonoRepo.Test.html#build_deps/0","title":"MonoRepo.Test.build_deps/0","type":"function"},{"doc":"Builds a list of paths to test directories of all the applications found in apps directory recursively. It means that in case an application is apps directory includes apps directory, all those apps&#39; test directories will be included.","ref":"MonoRepo.Test.html#build_test_paths/0","title":"MonoRepo.Test.build_test_paths/0","type":"function"},{"doc":"Builds a list of paths to test directories of all the applications found in apps directory recursively until it finds the first match. Behaves in simmilar fashion to it&#39;s /0 implementation.","ref":"MonoRepo.Test.html#build_test_paths/1","title":"MonoRepo.Test.build_test_paths/1","type":"function"}]